package main

import (
	"context"
	"fmt"
	"time"

	"github.com/go-telegram/bot"
	"github.com/go-telegram/bot/models"
)

func GetChatFullName(chat models.Chat) string {
	if chat.Title != "" {
		return chat.Title
	}
	if chat.FirstName != "" {
		if chat.LastName != "" {
			return fmt.Sprint(chat.FirstName, " ", chat.LastName)
		}
		return chat.FirstName
	}
	return ""
}

func GetUserFullName(user models.User) string {
	if user.FirstName != "" {
		if user.LastName != "" {
			return fmt.Sprint(user.FirstName, " ", user.LastName)
		}
		return user.FirstName
	}
	return ""
}

func FetchMemberFromChat(ctx context.Context, b *bot.Bot, chatID int64, userID int64) (*models.ChatMember, error) {
	return b.GetChatMember(ctx, &bot.GetChatMemberParams{ChatID: chatID, UserID: userID})
}

func IsAdmin(member models.ChatMember) bool {
	return member.Administrator != nil || member.Owner != nil
}

func DeleteAfterSeconds(chatID int64, msgID int, seconds int) error {
	return AddMessageToDelete(MessageToDelete{MessageId: msgID, ChatId: chatID, DeleteDate: time.Now().Add(time.Duration(seconds) * time.Second).Unix(), Tries: 0})
}
